<?php

namespace EventPortal\Domain\Agenda;

use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Agenda\Values\Session;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use Spatie\Period\Boundaries;
use Spatie\Period\Period;
use Spatie\Period\Precision;

class AgendasForDelegate
{
    public static function get(Delegate $delegate, Event $event): array
    {
        return Agenda::forEvent($event)
            ->orderBy('date')
            ->get()
            ->map(function (Agenda $agenda) use ($delegate): array {
                return [
                    'title' => $agenda->date->format('jS F'),
                    'active' => $agenda->date->isToday(),
                    'sessions' => self::calculateAgenda($agenda, $delegate),
                ];
            })
            ->values()
            ->toArray();
    }

    private static function calculateAgenda(Agenda $agenda, Delegate $delegate): array
    {
        $eventDelegate = EventDelegate::forEventAndDelegate($agenda->event, $delegate)->first();

        $defaultPathway = Pathway::query()
            ->with(['sessions' => function ($query) use ($agenda) {
                return $query->where('agenda_id', $agenda->id);
            }, 'sessions.eventSpeakers', 'sessions.eventVenue'])
            ->defaultForEvent($agenda->event)
            ->first();

        $nonDefaultPathway = Pathway::query()
            ->with(['sessions' => function ($query) use ($agenda) {
                return $query->where('agenda_id', $agenda->id);
            }, 'sessions.eventSpeakers', 'sessions.eventVenue'])
            ->nonDefaultForEventDelegate($eventDelegate)
            ->first();

        if ($nonDefaultPathway === null) {
            return $defaultPathway
                ->sessions
                ->sortBy('starting_at')
                ->mapInto(Session::class)
                ->values()
                ->toArray();
        }

        $nonDefaultPathwayPeriods = $nonDefaultPathway
            ->sessions
            ->map(function (Models\Session $session) {
                return new Period(
                    $session->starting_at->toDateTimeImmutable(),
                    $session->ending_at->toDateTimeImmutable(),
                    Precision::SECOND,
                    Boundaries::EXCLUDE_ALL
                );
            })
            ->toArray();

        $sessions = $defaultPathway
            ->sessions
            ->reject(function (Models\Session $session) use ($nonDefaultPathwayPeriods) {
                $period = new Period(
                    $session->starting_at->toDateTimeImmutable(),
                    $session->ending_at->toDateTimeImmutable(),
                    Precision::SECOND,
                    Boundaries::EXCLUDE_ALL
                );

                return $period->overlap(...$nonDefaultPathwayPeriods)->isEmpty() === false;
            });

        return $nonDefaultPathway
            ->sessions
            ->merge($sessions)
            ->sortBy('starting_at')
            ->mapInto(Session::class)
            ->values()
            ->toArray();
    }
}
