<?php

namespace EventPortal\Tests\Feature\Admin\Event;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Spatie\Snapshots\MatchesSnapshots;

class EventShowTest extends TestCase
{
    use DatabaseMigrations, MatchesSnapshots, SeedRolesAndPermissions;

    /** @test */
    public function can_view_event()
    {
        $event = factory(Event::class)->state('snapshot')->create();

        $this->assertMatchesJsonSnapshot(
            $this
                ->actingAsSuperAdmin()
                ->get(route('admin.event.show', $event))
                ->props()
        );
    }

    /** @test */
    public function can_view_event_but_not_edit(): void
    {
        config()->set('forms.event.published.can_edit', false);

        $event = factory(Event::class)->state('snapshot')->create(['status' => 'published']);

        $this->assertMatchesJsonSnapshot(
            $this
                ->actingAsSuperAdmin()
                ->get(route('admin.event.show', $event))
                ->props()
        );
    }
}
